; ------------------------------------------------------------------
; --  _____       ______  _____                                    -
; -- |_   _|     |  ____|/ ____|                                   -
; --   | |  _ __ | |__  | (___    Institute of Embedded Systems    -
; --   | | | '_ \|  __|  \___ \   Zurich University of             -
; --  _| |_| | | | |____ ____) |  Applied Sciences                 -
; -- |_____|_| |_|______|_____/   8401 Winterthur, Switzerland     -
; ------------------------------------------------------------------
; --
; -- main.s
; --
; -- CT1 P08 "Strukturierte Codierung" mit Assembler
; --
; -- $Id: struct_code.s 3787 2016-11-17 09:41:48Z kesr $
; ------------------------------------------------------------------
;Directives
        PRESERVE8
        THUMB

; ------------------------------------------------------------------
; -- Address-Defines
; ------------------------------------------------------------------
; input
ADDR_DIP_SWITCH_7_0       EQU        0x60000200
ADDR_BUTTONS              EQU        0x60000210

; output
ADDR_LED_31_0             EQU        0x60000100
ADDR_7_SEG_BIN_DS3_0      EQU        0x60000114
ADDR_LCD_COLOUR           EQU        0x60000340
ADDR_LCD_ASCII            EQU        0x60000300
ADDR_LCD_ASCII_BIT_POS    EQU        0x60000302
ADDR_LCD_ASCII_2ND_LINE   EQU        0x60000314


; ------------------------------------------------------------------
; -- Program-Defines
; ------------------------------------------------------------------
; value for clearing lcd
ASCII_DIGIT_CLEAR        EQU         0x00000000
LCD_LAST_OFFSET          EQU         0x00000028

; offset for showing the digit in the lcd
ASCII_DIGIT_OFFSET        EQU        0x00000030

; lcd background colors to be written
DISPLAY_COLOUR_RED        EQU        0
DISPLAY_COLOUR_GREEN      EQU        2
DISPLAY_COLOUR_BLUE       EQU        4

; ------------------------------------------------------------------
; -- myConstants
; ------------------------------------------------------------------
        AREA myConstants, DATA, READONLY
; display defines for hex / dec
DISPLAY_BIT               DCB        "Bit "
DISPLAY_2_BIT             DCB        "2"
DISPLAY_4_BIT             DCB        "4"
DISPLAY_8_BIT             DCB        "8"
        ALIGN

; ------------------------------------------------------------------
; -- myCode
; ------------------------------------------------------------------
        AREA myCode, CODE, READONLY
        ENTRY

        ; imports for calls
        import adc_init
        import adc_get_value

main    PROC
        export main
        ; 8 bit resolution, cont. sampling
        BL         adc_init 
        BL         clear_lcd

main_loop
; STUDENTS: To be programmed

		; Clear LCD
		BL			clear_lcd_colour
		
		; Read ADC Value to R0
		BL			adc_get_value
		
		; check T0
		LDR			R1, =ADDR_BUTTONS
		LDRB		R1, [R1]
		LDR			R2, =0x01
		TST     	R1, R2
		BNE			case_green					; If T0 is pressed: goto case_green, else: continue

		; Else (T0 is not pressed):
		
		; Read 8-bit value from DIP-Switch 7-0 to R1
		LDR			R1, =ADDR_DIP_SWITCH_7_0
		LDRB		R1, [R1]
		
		; Calc diff: DIP-Switch-7-0 - ADC-Value
		SUBS		R2, R1, R0
		BLT			case_red					; IF diff >= 0: continue case blue, else: goto case red
		

case_blue
		; Make LCD Display Blue
		LDR			R3, =ADDR_LCD_COLOUR
		LDR			R4, =DISPLAY_COLOUR_BLUE
		LDR			R5, =0xffff
		STRH		R5, [R3, R4]
		
		; Show Diff on 7-Segment
		LDR			R3, =ADDR_7_SEG_BIN_DS3_0
		STRB		R2, [R3]
		
		; Show Bits on lcd Display
		LDR			R3, =0x8
		SUBS		R3, R3, R2
		BLS			case_8_bit
		LDR			R3, =0x4
		SUBS		R3, R3, R2
		BLS			case_4_bit
case_2_bit
		LDR        R4, =DISPLAY_2_BIT
		B			end_case
case_4_bit
		LDR        R4, =DISPLAY_4_BIT
		B			end_case
case_8_bit
		LDR        R4, =DISPLAY_8_BIT
		
end_case
		LDR        R3, =ADDR_LCD_ASCII
        
        LDR        R4, [R4]
        STRB       R4, [R3]
		
		BL			write_bit_ascii

		B          main_loop
case_red
		; Make LCD Display Red
		LDR			R3, =ADDR_LCD_COLOUR
		LDR			R4, =DISPLAY_COLOUR_RED
		LDR			R5, =0xffff
		STRH		R5, [R3, R4]
		
		; Show Diff on 7-Segment
		LDR			R3, =ADDR_7_SEG_BIN_DS3_0
		STRB		R2, [R3]
		
		; Count 0 Bits in diff
		LDR			R3, =0x9		; Count down
		LDR			R4, =0x0		; Count 0 bits
		LDR			R5, =0x01		; Value 1
loop_count_0_bits
		SUBS		R3, R3, R5
		BEQ			end_loop_count_0_bits
		LSRS     	R2, R2, #1
		BHS			loop_count_0_bits
		ADDS		R4, R4, R5
		B			loop_count_0_bits
end_loop_count_0_bits

		; Show 0 Bits in lcd 2nd line
		LDR         R3, =ADDR_LCD_ASCII_2ND_LINE
		LDR			R5, =ASCII_DIGIT_OFFSET
		ADDS		R4, R4, R5
		STRB		R4, [R3]

		B          main_loop
case_green
		; Make LCD Display Green
		LDR			R3, =ADDR_LCD_COLOUR
		LDR			R4, =DISPLAY_COLOUR_GREEN
		LDR			R5, =0xffff
		STRH		R5, [R3, R4]
		
		; Show ADC Value on 7-Semgment
		LDR			R3, =ADDR_7_SEG_BIN_DS3_0
		STRB		R0, [R3]
		
		LDR			R1, =0x0			; Register to show on LEDs
		LDR			R2, =0x1			; Value 0x1
		LSRS		R0, R0, #3			; Counter
led_loop
		BEQ			end_led_loop
		LSLS		R1, R2
		ORRS		R1, R1, R2
		SUBS		R0, R0, R2
		B			led_loop
end_led_loop
		LDR			R0, =ADDR_LED_31_0
		STR			R1, [R0]
		

; END: To be programmed
        B          main_loop
		
clear_lcd_colour
		PUSH		{R0, R1, R2}
		LDR			R0, =ADDR_LCD_COLOUR
		LDR			R2, =0x0
		LDR			R1, =DISPLAY_COLOUR_RED
		STRH		R2, [R0, R1]
		LDR			R1, =DISPLAY_COLOUR_BLUE
		STRH		R2, [R0, R1]
		LDR			R1, =DISPLAY_COLOUR_GREEN
		STRH		R2, [R0, R1]
		POP			{R0, R1, R2}
		BX			LR
        
clear_lcd
        PUSH       {R0, R1, R2}
        LDR        R2, =0x0
clear_lcd_loop
        LDR        R0, =ADDR_LCD_ASCII
        ADDS       R0, R0, R2                       ; add index to lcd offset
        LDR        R1, =ASCII_DIGIT_CLEAR
        STR        R1, [R0]
        ADDS       R2, R2, #4                       ; increas index by 4 (word step)
        CMP        R2, #LCD_LAST_OFFSET             ; until index reached last lcd point
        BMI        clear_lcd_loop
        POP        {R0, R1, R2}
        BX         LR

write_bit_ascii
        PUSH       {R0, R1}
        LDR        R0, =ADDR_LCD_ASCII_BIT_POS 
        LDR        R1, =DISPLAY_BIT
        LDR        R1, [R1]
        STR        R1, [R0]
        POP        {R0, R1}
        BX         LR

        ENDP
        ALIGN


; ------------------------------------------------------------------
; End of code
; ------------------------------------------------------------------
        END
